;   /-----------------------------------------/
;  /--Multiline Statusbar by Dave, 25.09.13--/
; /-----------------------------------------/

EnableExplicit

Enumeration ;Feld Art
  #Feld_Text
  #Feld_Image
  #Feld_Progressbar
EndEnumeration

Enumeration ;Textposition
  #TextPos_Links
  #TextPos_Mitte
  #TextPos_Rechts
EndEnumeration


Structure _Feld
  X.l
  Y.l
  Breite.l
  Hoehe.l
  Art.l
  Text$
  Wert.l
  Max.l
  Image.i
  Farbe_Text.l
  Farbe_Processbar.l
  TextPos.l
EndStructure

Structure _Zeile
  List Felder._Feld()
EndStructure

Structure _Statusbar
  X.l
  Y.l
  Breite.l
  Hoehe.l
  Window.i
  Canvas.i
  ZeilenHoehe.l
  Font.i
  Farbe_Raender.l
  Array Zeilen._Zeile(0)
EndStructure



Procedure SB_Erstellen(Window, Zeilen = 1, ZeilenHoehe = 20, Randfarbe = $BEBEBE)
  Protected *SB._Statusbar
  
  *SB = AllocateMemory(SizeOf(_Statusbar))
  If *SB
    InitializeStructure(*SB, _Statusbar)
    
    With *SB
      \Breite = WindowWidth(Window)
      \Hoehe = Zeilen * ZeilenHoehe
      \X = 0
      \Y = WindowHeight(Window)-\Hoehe
      \Window = Window
      \ZeilenHoehe = ZeilenHoehe
      \Farbe_Raender = Randfarbe
      Dim \Zeilen(Zeilen-1)
      \Font = LoadFont(#PB_Any, "Verdana", ZeilenHoehe/2)
      \Canvas = CanvasGadget(#PB_Any, \X, \Y, \Breite, \Hoehe)
    EndWith
    
    ProcedureReturn *SB
  EndIf
EndProcedure


Procedure SB_Hoehe(*SB._Statusbar)
  If *SB
    ProcedureReturn *SB\Hoehe
  EndIf
EndProcedure


Procedure SB_Free(*SB._Statusbar)
  Protected Zeile.l
  
  If *SB
    With *SB
      For Zeile = 0 To ArraySize(\Zeilen()) 
        FreeList(\Zeilen(Zeile)\Felder())
      Next
      FreeArray(\Zeilen())
      FreeGadget(\Canvas)
      FreeFont(\Font)
      FreeMemory(*SB)
    EndWith
  EndIf
EndProcedure


Procedure SB_Aktualisieren(*SB._Statusbar)
  Protected.l Zeile, PosX, TextX
  Protected.f Zoom1, Zoom2, Zoom, Breite
  
  If *SB 
    ResizeGadget(*SB\Canvas, #PB_Ignore, WindowHeight(*SB\Window)-*SB\Hoehe, WindowWidth(*SB\Window), #PB_Ignore)
    *SB\Breite = WindowWidth(*SB\Window)
    If StartDrawing(CanvasOutput(*SB\Canvas))
      Box(0, 0, *SB\Breite, *SB\Hoehe, $F0F0F0)
      DrawingMode(#PB_2DDrawing_Transparent)
      DrawingFont(FontID(*SB\Font))
      
      With *SB\Zeilen(Zeile)\Felder()
        
        For Zeile = 0 To ArraySize(*SB\Zeilen())
          Line(0, Zeile * *SB\ZeilenHoehe, *SB\Breite, 1, *SB\Farbe_Raender)
          PosX = 0
          
          ForEach *SB\Zeilen(Zeile)\Felder()
            \X = PosX
            PosX + \Breite+3
            Line(\X+\Breite+1, \Y+3, 1, \Hoehe-5, *SB\Farbe_Raender)
            Box(\X, \Y+1, \Breite, \Hoehe-1, $F0F0F0)
            Select \TextPos
              Case #TextPos_Links  : TextX = \X +5
              Case #TextPos_Mitte  : TextX = \X + \Breite/2 - TextWidth(\Text$)/2
              Case #TextPos_Rechts : TextX = \X + (\Breite-5) -TextWidth(\Text$)
            EndSelect
            
            Select \Art
                
              Case #Feld_Text    
                DrawText(TextX, \Y+\Hoehe/2-TextHeight("H")/2, \Text$, \Farbe_Text)
                
              Case #Feld_Image
                If IsImage(\Image)
                  If ImageWidth(\Image) > \Breite Or ImageHeight(\Image) > \Hoehe-1
                    Zoom1 = \Breite / ImageWidth(0) 
                    Zoom2 = (\Hoehe-1)  / ImageHeight(0)
                    If Zoom1 > Zoom2 : Zoom = Zoom2 :Else: Zoom = Zoom1 : EndIf
                  EndIf
                  DrawImage(ImageID(\Image), \X+\Breite/2-(ImageWidth(\Image)/2) * Zoom, \Y+1+\Hoehe/2-(ImageHeight(\Image)/2) * Zoom, ImageWidth(\Image) * Zoom, ImageHeight(\Image) * Zoom)
                EndIf
                
              Case #Feld_Progressbar
                RoundBox(\X+1, \Y+2, \Breite-2, \Hoehe-3, 4, 4, $B9B9B9)
                Breite = (\Breite-2)/\Max * \Wert
                RoundBox(\X+2, \Y+3, Breite, \Hoehe-5, 4, 4, \Farbe_Processbar)  
                DrawText(TextX, \Y+\Hoehe/2-TextHeight("H")/2, \Text$, \Farbe_Text)
                
            EndSelect
          Next
        Next
      EndWith
      
      StopDrawing()
    EndIf 
  EndIf
EndProcedure


Procedure SB_NeuesFeld(*SB._Statusbar, Breite, Zeile = 1)
  Protected Art.l
  
  If *SB
    With *SB\Zeilen(Zeile-1)
      LastElement(\Felder())
      AddElement(\Felder())
      \Felder()\Art = Art
      \Felder()\Breite = Breite
      \Felder()\Hoehe = *SB\ZeilenHoehe
      \Felder()\Y = (Zeile-1) * *SB\ZeilenHoehe
      SB_Aktualisieren(*SB)
    EndWith
  EndIf
EndProcedure


Procedure SB_Text(*SB._Statusbar, Feld, Text$, Zeile = 1, TextPos = #TextPos_Links, Textfarbe = $050505)
  If *SB
    With *SB\Zeilen(Zeile-1)
      SelectElement(\Felder(), Feld)
      \Felder()\Art = #Feld_Text
      \Felder()\Text$ = Text$
      \Felder()\Farbe_Text = Textfarbe
      \Felder()\TextPos = TextPos
      SB_Aktualisieren(*SB)
    EndWith
  EndIf
EndProcedure


Procedure SB_Image(*SB._Statusbar, Feld, Image, Zeile = 1)
  If *SB
    With *SB\Zeilen(Zeile-1)
      SelectElement(\Felder(), Feld)
      \Felder()\Art = #Feld_Image
      \Felder()\Image = Image
      SB_Aktualisieren(*SB)
    EndWith
  EndIf
EndProcedure


Procedure SB_Progress(*SB._Statusbar, Feld, Wert, Text$ = "", Max = 100, Zeile = 1, TextPos = #TextPos_Links, ProgressbarFarbe = $06B025, Textfarbe = $050505)
  If *SB
    With *SB\Zeilen(Zeile-1)
      SelectElement(\Felder(), Feld)
      \Felder()\Art = #Feld_Progressbar
      \Felder()\Wert = Wert
      \Felder()\Text$ = Text$
      \Felder()\Max = Max
      \Felder()\Farbe_Text = Textfarbe
      \Felder()\Farbe_Processbar = ProgressbarFarbe
      \Felder()\TextPos = TextPos
      SB_Aktualisieren(*SB)
    EndWith
  EndIf
EndProcedure


; IDE Options = PureBasic 5.00 (Windows - x64)
; CursorPosition = 103
; FirstLine = 46
; Folding = --
; EnableXP
; CPU = 1